import logging
import subprocess

logger = logging.getLogger(__name__)

apps = ["eab_base", "student-path", "notification-service", "greendale"]


def change_reqs_files(app, args):
    if app == "eab_base":
        return

    if app in "greendale":
        files_cmd = "find $PROJECTS/{app}/requirements -name 'remote.txt'".format(app=app)
    else:
        if args.all_reqs:
            files_cmd = "find $PROJECTS/{app}/requirements -name '*.txt'".format(app=app)
        else:
            files_cmd = "find $PROJECTS/{app}/requirements -name 'local.txt' -o  -name 'remote.txt'".format(app=app)

    files_to_change = filter(
        lambda x: x != "",
        subprocess.check_output(files_cmd, shell=True).decode('utf-8').split('\n')
    )

    for file_to_change in files_to_change:
        logger.info('Changing {file}'.format(file=file_to_change))

        cmd = "sed -i \'\' -E \"s|(git@github\.com/advisory/eab_base\.git@)[^\#]+|\\1{br}|g\" {file}".format(
            br=args.branch_name,
            file=file_to_change)
        subprocess.call(cmd, shell=True)

        cmd = "sed -i \'\' -E \"s|(git@github\.com/advisory/student-path\.git@)[^\#]+|\\1{br}|g\" {file}".format(
            br=args.branch_name,
            file=file_to_change)
        subprocess.call(cmd, shell=True)


def add_and_commit(app, msg):
    cmd = "git -C $PROJECTS/{app} add .".format(app=app)
    subprocess.call(cmd, shell=True)

    cmd = 'git -C $PROJECTS/{app} commit -m "{0}"'.format(app=app, msg=msg)
    subprocess.call(cmd, shell=True)


def git_exec(app, cmd):
    cmd = "git -C $PROJECTS/{app} {cmd}".format(app=app, cmd=cmd)
    subprocess.call(cmd, shell=True)


def simple_slugify(name):
    return name.lower().strip().replace(' ', '-')


def branch_name(args):
    return "{ticket_type}/{ticket_id}-{ticket_name}".format(ticket_name=simple_slugify(args.ticket_name),
                                                            ticket_type=args.ticket_type, ticket_id=args.ticket_id)



import unittest
import os
import json
from mock import patch
from .api import get_or_create_pull_request
import logging
logger = logging.getLogger(__name__)

__location__ = os.path.realpath(os.path.join(os.getcwd(), os.path.dirname(__file__)))


with open(os.path.join(__location__, 'json_samples/already_exists_created.json')) as f:
    data = f.read()
    already_exists_json = json.loads(data)

with open(os.path.join(__location__, 'json_samples/valid_created.json')) as f:
    data = f.read()
    created = json.loads(data)

with open(os.path.join(__location__, 'json_samples/valid_get.json')) as f:
    data = f.read()
    valid_get = json.loads(data)


def mocked_requests_post_created(*args, **kwargs):
    class MockResponse:
        def __init__(self, json_data, status_code):
            self.json_data = json_data
            self.content = json_data
            self.status_code = status_code

        def json(self):
            return self.json_data
    return MockResponse(created, 201)


def mocked_requests_get_valid(*args, **kwargs):
    class MockResponse:
        def __init__(self, json_data, status_code):
            self.json_data = json_data
            self.content = json_data
            self.status_code = status_code

        def json(self):
            return self.json_data
    return MockResponse(valid_get, 201)


def mocked_requests_get_invalid(*args, **kwargs):
    class MockResponse:
        def __init__(self, json_data, status_code):
            self.json_data = json_data
            self.content = json_data
            self.status_code = status_code

        def json(self):

            return self.json_data
    return MockResponse(valid_get, 400)


class TestGetOrCreateBranch(unittest.TestCase):

    @patch('requests.post', side_effect=mocked_requests_post_created)
    @patch('requests.get', side_effect=mocked_requests_get_invalid)
    def test_create(self, mock_get, mock_post):
        params = {
            "head": "test-branch",
            "base": "develop"
        }
        response_json = get_or_create_pull_request('foo.com', 'bwarren2', params)
        self.assertEqual(response_json['id'], 147363129)

    @patch('requests.get', side_effect=mocked_requests_get_valid)
    def test_get(self, mock_get):
        params = {
            "head": "test-branch",
            "base": "develop"
        }
        response_json = get_or_create_pull_request('foo.com', 'bwarren2', params)
        self.assertEqual(response_json['id'], 147611694)



import subprocess
import readline
if 'libedit' in readline.__doc__:
    readline.parse_and_bind("bind ^I rl_complete")
else:
    readline.parse_and_bind("tab: complete")


branches = subprocess.check_output("git  branch --remote --sort=-committerdate", shell=True).decode('utf-8').split('\n')
branches = filter(lambda x: x, [x.replace(',', '').replace('origin/', '').strip() for x in branches])


def completer(text, state):
    options = [x for x in branches if x.startswith(text)]
    try:
        return options[state]
    except IndexError:
        return None


readline.set_completer(completer)
readline.parse_and_bind("tab: complete")

inp = raw_input("Pick a branch: ")
print "You entered", inp



import subprocess


class Password(object):

    def __str__(self):
        """ Get the user password from the keychain or fail."""

        command = '/usr/bin/security find-generic-password -wl abcemployees'.split(' ')
        process = subprocess.Popen(command, stdout=subprocess.PIPE)
        password = process.stdout.read().strip()
        if password is None:
            raise RuntimeError(
                'We need your abcemployees password on your keychain, or specify one via JIRA_PASSWD as an env var.'
            )

        return password



#!/usr/bin/env python

import logging
import argparse
from collections import OrderedDict
import os
from .tickets.jira import *
from .tickets.branches import *
from .tickets.completer import branch_autocomplete
from .tickets import github

DEFAULT_BASE = 'develop'

logger = logging.getLogger(__name__)


def process(app_bases, args):
    logger.info("Starting")
    logger.info("Making a ticket")

    # ticket = make_ticket(args)

    logger.info("Making branches...")
    for app in apps:
        logger.info("Making branch for for {0}".format(app))
        git_exec(app, 'checkout {0}'.format(app_bases[app]))
        git_exec(app, "checkout -b {branch_name}".format(branch_name=args.branch_name))

        if app != 'eab_base':
            logger.info("Changing requirements files")
            change_reqs_files(app, args)
            git_exec(app, 'add $PROJECTS/{app}/requirements'.format(app=app))
            git_exec(app, 'commit -m "Change reqs"')
            logger.info("Committing")

        if args.force_pr:
            cmd = 'touch $PROJECTS/{app}/fakemagicfile.sh'.format(app=app)
            subprocess.call(cmd, shell=True)
            git_exec(app, 'add $PROJECTS/{app}/fakemagicfile.sh'.format(app=app))
            git_exec(app, 'commit -m "Make a fake file"')
            cmd = 'rm $PROJECTS/{app}/fakemagicfile.sh'.format(app=app)
            subprocess.call(cmd, shell=True)
            git_exec(app, 'rm $PROJECTS/{app}/fakemagicfile.sh'.format(app=app))
            git_exec(app, 'commit -m "Delete a fake file"')

    logger.info("Pushing to remote")
    for app in apps:
        git_exec(app, 'push --set-upstream origin {br}'.format(app=app, br=args.branch_name))

    prs = OrderedDict()
    logger.info("Making PRs")
    for app, url in github.pr_urls.items():
        logger.info("Making PRs for {0}".format(app))
        params = {
            'head': args.branch_name,
            'base': app_bases[app],
        }
        prs[app] = github.get_or_create_pull_request(url, params, args)

    messages = [ticket_url(args.ticket_id)]
    messages.extend(["{0}: {1}".format(project, x['html_url']) for project, x in prs.items()])
    message_str = "\n".join(messages)
    logger.info("Composed the message string")
    for app, pr in prs.items():
        logger.info("Updating the message for {0}".format(pr))
        github.update_pr("{0}/{1}".format(github.pr_urls[app], pr['number']), message_str, args)

    # logger.info("Updating the message for the ticket")
    # update_ticket(args)
    logger.info("Done!")
    # for proj, x in prs.items(): print(proj, x['html_url'])
    # ('eab_base', u'https://github.com/advisory/eab_base/pull/1502')
    # ('student-path', u'https://github.com/advisory/student-path/pull/823')
    # ('greendale', u'https://github.com/advisory/greendale/pull/2212')
    # ('notification-service', u'https://github.com/advisory/notification-service/pull/261')
    for proj, x in prs.items():
        print("{0}: {1}".format(proj, x['html_url']))


def main():

    parser = argparse.ArgumentParser()
    parser.add_argument('--all-reqs', '-l', action='store_true', default=False)
    parser.add_argument('--ticket_name', '-b', help="Branch name to use without story suffix", required=True)
    parser.add_argument('--ticket-id', '-t', help='Ticket name on JIRA, ex NAV-5050', required=True)
    parser.add_argument('--ticket-type', '-type', help='Type of ticket.  Default feature.', default='feature')
    parser.add_argument('---pick-base-branch', '-pb', action='store_true',
                        help='Interactively choose your base branch, instead of using develop.')
    # parser.add_argument('--exclude', '-x', action='store',
    #                     help='Exclude these apps from operation. Ex `greendale,notification_service`')
    parser.add_argument('--force-pr', '-f', action='store_true',
                        help='Make sure a PR gets made by making a noop commit and reverting it. (Default True)',
                        default=True)
    parser.add_argument("-v", "--verbose", help="increase output verbosity", action="store_true")
    args = parser.parse_args()

    if args.verbose:
        logging.basicConfig(level=logging.DEBUG)

    args.user = github.get_user()
    args.branch_name = branch_name(args)

    GITHUB_TOKEN = os.getenv('GITHUB_TOKEN')

    if not GITHUB_TOKEN:
        raise RuntimeError("You need a GITHUB_TOKEN env var with your github token to use this tool.")

    logging.info("Getting app base branches")
    app_bases = {}
    for app in apps:
        if args.pick_base_branch:
            base_branch = branch_autocomplete(app)
            app_bases[app] = base_branch
        else:
            app_bases[app] = DEFAULT_BASE
    process(app_bases, args)



import unittest
from .branches import simple_slugify, branch_name
from collections import namedtuple


class TestSlugifier(unittest.TestCase):

    def test_stripped(self):
        self.assertEqual(simple_slugify('  foo '), 'foo')

    def test_replaced(self):
        self.assertEqual(simple_slugify('foo bar'), 'foo-bar')

    def test_lowered(self):
        self.assertEqual(simple_slugify('FOO BaR'), 'foo-bar')

    def test_all(self):
        self.assertEqual(simple_slugify('   FOO BaR '), 'foo-bar')


class TestBranchName(unittest.TestCase):

    def test_formatted(self):
        Args = namedtuple('Args', ['ticket_type', 'ticket_id', 'ticket_name'])
        args = Args('feature', 'NAV-5050', 'Do a thing')

        self.assertEqual(branch_name(args), 'feature/NAV-5050-do-a-thing')



import os
from setuptools import setup, find_packages

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="pr-creator",
    version="0.0.10",
    author="Ben Warren",
    author_email="bwarren@eab.com",
    description=("Git lifestyle utility for NaviGuide"),
    license="BSD",
    keywords="happy dev",
    url="https://github.com/bwarren2/create-prs",
    packages=find_packages(),
    long_description=read('README.md'),
    classifiers=[
        "Topic :: Utilities",
        "License :: OSI Approved :: BSD License",
    ],
    entry_points={
        "console_scripts": [
            "create-prs=pr_creator.main:main",
        ]
    },
    install_requires=[
        'requests>=2.18.4',
    ],
)



from os import getenv
import requests
from requests.auth import HTTPBasicAuth
from .password import Password

JIRA_BASE_URL = 'https://workflow.advisory.com'
JIRA_PASSWD = getenv('JIRA_PASSWD', Password())
JIRA_USER = getenv('JIRA_USER', getenv('USER'))


def make_ticket(args):
    payload = {
        "fields": {
            "project": {
                "key": "NAV"
            },
            "summary": args.ticket_id,
            "customfield_10327": "{0} needs to write an acceptance.".format(args.user),
            "customfield_13600": "{0} needs to write a story.".format(args.user),
            "issuetype": {
                "name": "Story"
            }
        }
    }
    resp = requests.post(
        "{0}/rest/api/2/issue/".format(JIRA_BASE_URL),
        auth=HTTPBasicAuth(JIRA_USER, JIRA_PASSWD),
        json=payload
    )
    # returns {"id":"315527","key":"NAV-5467","self":"https://workflow.advisory.com/rest/api/2/issue/315527"}
    if resp.status_code in (200, 201):
        resp_data = resp.json()
        resp_data['access_url'] = ticket_url(args.ticket_id)
        return resp_data
    else:
        raise RuntimeError("Something went wrong talking to jira. {0}".format(dir(resp)))


def ticket_url(key):
    return "https://workflow.advisory.com/browse/{0}".format(key)



import subprocess
import readline
if 'libedit' in readline.__doc__:
    readline.parse_and_bind("bind ^I rl_complete")
else:
    readline.parse_and_bind("tab: complete")


def branch_autocomplete(app):
    branches = subprocess.check_output("git -C $PROJECTS/{0} branch --remote --sort=-committerdate".format(app),
                                       shell=True).decode('utf-8').split('\n')
    branches = filter(lambda x: x, [x.replace(',', '').replace('origin/', '').strip() for x in branches])

    def completer(text, state):
        options = [x for x in branches if x.startswith(text)]
        try:
            return options[state]
        except IndexError:
            return None

    readline.set_completer(completer)
    readline.parse_and_bind("tab: complete")
    # Unfortunately, we can't sort the completer output correctly without a big lift.
    # See https://pewpewthespells.com/blog/osx_readline.html.

    inp = ""
    while inp not in branches:
        inp = raw_input("Pick a branch for {0} (tab to complete): ".format(app))

    return inp



import re
import subprocess
import requests
from requests.auth import HTTPBasicAuth
from collections import OrderedDict
from ..settings import GITHUB_TOKEN

pr_urls = OrderedDict()
pr_urls['eab_base'] = "https://api.github.com/repos/advisory/eab_base/pulls"
pr_urls['student-path'] = "https://api.github.com/repos/advisory/student-path/pulls"
pr_urls['greendale'] = "https://api.github.com/repos/advisory/greendale/pulls"
pr_urls['notification-service'] = "https://api.github.com/repos/advisory/notification-service/pulls"


def get_user():
    regex = re.compile("(?<=Hi )(.*)(?=! You've)")

    try:
        subprocess.check_output("ssh -T git@github.com 2>&1", shell=True)
    except subprocess.CalledProcessError as exc:
        output = exc.output.decode('utf-8')

        if "You've successfully authenticated, but GitHub does not provide shell access." in output:
            # We were actually successful
            return regex.search(output).group(1)
        else:
            raise


def get_or_create_pull_request(url, params, args):

    payload = {
        "title": args.branch_name,
        "body": "Placeholder",
    }
    payload.update(params)

    payload['head'] = "advisory:{0}".format(payload['head'])

    # Check for existence
    resp = requests.get(
        url,
        auth=HTTPBasicAuth(args.user, GITHUB_TOKEN),
        params=payload
    )

    if resp.status_code in (200, 201):
        # PR already exists
        if len(resp.json()) == 1:
            return resp.json()[0]
        elif len(resp.json()) > 1:
            raise RuntimeError("Too many PRs exist for url: {0}, user: {1}, params: {2}".format(
                url, args.user, params))

    # We need to create
    resp = requests.post(
        url,
        auth=HTTPBasicAuth(args.user, GITHUB_TOKEN),
        json=payload
    )

    if resp.status_code in (200, 201):
        return resp.json()
    else:
        raise RuntimeError("We couldn't get or create the PR for url: {0}, user: {1}, params: {2}".format(
            url, args.user, params))


def update_pr(url, body, args):

    payload = {
        "body": body,
    }

    # Check for existence
    resp = requests.patch(
        url,
        auth=HTTPBasicAuth(args.user, GITHUB_TOKEN),
        json=payload
    )

    if resp.status_code not in (200, 201):
        # PR already exists
        raise RuntimeError("Couldn't update PR {0}".format(url))
